﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.Reflection;

namespace howto_reflection_list_methods
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // Add some methods.
        private void MyPrivateMethod(string arg0, string arg2) { }
        public void MyPublicMethod(float float0, double double0) { }
        private void MyOverloadedMethod() { }
        private void MyOverloadedMethod(int arg0) { }
        protected void MyProtectedMethod() { }

        private void Form1_Load(object sender, EventArgs e)
        {
            // Make column headers.
            lvwMethods.Columns.Clear();
            lvwMethods.Columns.Add("Method", 200,
                HorizontalAlignment.Left);
            lvwMethods.Columns.Add("Description", 300,
                HorizontalAlignment.Left);
            lvwMethods.Columns.Add("Properties", 150,
                HorizontalAlignment.Left);

            // List the methods.
            // Use the class you want to study instead of Form1.
            MethodInfo[] method_infos = typeof(Form1).GetMethods(
                BindingFlags.FlattenHierarchy |
                BindingFlags.Instance |
                BindingFlags.NonPublic |
                BindingFlags.Public |
                BindingFlags.Static);
            foreach (MethodInfo info in method_infos)
            {
                // Make a list of properties.
                string properties = "";
                if (info.IsConstructor) properties += " ctor";
                if (info.IsAbstract) properties += " abstract";
                if (info.IsPrivate) properties += " private";
                if (info.IsPublic) properties += " public";
                if (info.IsFamily) properties += " protected";
                if (info.IsFamilyAndAssembly) properties += " protected AND internal";
                if (info.IsFamilyOrAssembly) properties += " protected OR internal";
                if (info.IsFinal) properties += " sealed";
                if (info.IsGenericMethod) properties += " generic";
                if (info.IsStatic) properties += " static";
                if (info.IsVirtual) properties += " virtual";

                if (properties.Length > 0) properties = properties.Substring(1);
                ListViewMakeRow(lvwMethods,
                    info.Name,
                    info.ToString(),
                    properties);
            }

            // Size the columns to fit the data.
            for (int i = 0; i < lvwMethods.Columns.Count; i++)
                lvwMethods.Columns[i].Width = 150;

            // Size the form.
            int new_wid = 30;
            for (int i = 0; i < lvwMethods.Columns.Count; i++)
                new_wid += lvwMethods.Columns[i].Width;
            this.Size = new Size(new_wid, this.Size.Height);
        }

        // Make a ListView row.
        private void ListViewMakeRow(ListView lvw, String item_title, params string[] subitem_titles)
        {
            // Make the item.
            ListViewItem new_item = lvw.Items.Add(item_title);

            // Make the sub-items.
            for (int i = subitem_titles.GetLowerBound(0); i <= subitem_titles.GetUpperBound(0); i++)
            {
                new_item.SubItems.Add(subitem_titles[i]);
            }
        }
    }
}
